import resolve from 'rollup-plugin-node-resolve'
import buble from 'rollup-plugin-buble';
import copy from 'rollup-plugin-copy';
import includePaths from 'rollup-plugin-includepaths';
import license from 'rollup-plugin-license';
import { uglify } from "rollup-plugin-uglify";

export default {
  // dest: 'dist/js/app.js',
  input: 'src/js/index.js',
  output: {
    file: 'dist/js/app.min.js',
    name: 'WBADV',
    format: 'umd',
    sourceMap: true
  },
  plugins: [
    includePaths({
      paths: ['src/js', 'src/js/widget', 'src/js/misc', 'src/js/component',
        'src/js/page', 'src/js/dialog', 'src/js/arvados/base',
        'src/js/arvados/collection', 'src/js/arvados/process']
    }),
    copy({
      'src/html/index.html': 'dist/index.html',
      'src/css/index.css': 'dist/css/index.css',
      'node_modules/bootstrap/dist/css/bootstrap.min.css': 'dist/css/bootstrap.min.css',
      'node_modules/bootstrap/dist/js/bootstrap.min.js': 'dist/js/bootstrap.min.js',
      'node_modules/jquery/dist/jquery.min.js': 'dist/js/jquery.min.js',
      'node_modules/@fortawesome/fontawesome-free/js/fontawesome.min.js': 'dist/js/fontawesome.min.js',
      'node_modules/@fortawesome/fontawesome-free/css/all.min.css': 'dist/css/all.min.css',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-regular-400.eot': 'dist/webfonts/fa-regular-400.eot',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-regular-400.svg': 'dist/webfonts/fa-regular-400.svg',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-regular-400.ttf': 'dist/webfonts/fa-regular-400.ttf',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-regular-400.woff': 'dist/webfonts/fa-regular-400.woff',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-regular-400.woff2': 'dist/webfonts/fa-regular-400.woff2',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-solid-900.eot': 'dist/webfonts/fa-solid-900.eot',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-solid-900.svg': 'dist/webfonts/fa-solid-900.svg',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-solid-900.ttf': 'dist/webfonts/fa-solid-900.ttf',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-solid-900.woff': 'dist/webfonts/fa-solid-900.woff',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-solid-900.woff2': 'dist/webfonts/fa-solid-900.woff2',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-brands-400.eot': 'dist/webfonts/fa-brands-400.eot',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-brands-400.svg': 'dist/webfonts/fa-brands-400.svg',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-brands-400.ttf': 'dist/webfonts/fa-brands-400.ttf',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-brands-400.woff': 'dist/webfonts/fa-brands-400.woff',
      'node_modules/@fortawesome/fontawesome-free/webfonts/fa-brands-400.woff2': 'dist/webfonts/fa-brands-400.woff2',
      'node_modules/js-uuid/js-uuid.js': 'dist/js/js-uuid.js',
      'node_modules/filesize/lib/filesize.js': 'dist/js/filesize.js',
      'node_modules/crypto-js/core.js': 'dist/js/crypto-js/core.js',
      'node_modules/crypto-js/md5.js': 'dist/js/crypto-js/md5.js',
      'src/js/worker/wb-manifest-worker.js': 'dist/js/wb-manifest-worker.js',
      'node_modules/js-yaml/dist/js-yaml.min.js': 'dist/js/js-yaml.min.js',
      'node_modules/streamsaver/mitm.html': 'dist/mitm.html',
      'node_modules/streamsaver/sw.js': 'dist/sw.js',
      'src/js/thirdparty/StreamSaver.js': 'dist/js/StreamSaver.js',
      'node_modules/web-streams-polyfill/dist/ponyfill.js': 'dist/js/web-streams-polyfill/ponyfill.js',
      'node_modules/papaya-viewer/release/current/standard/papaya.js': 'dist/js/papaya.js',
      'node_modules/papaya-viewer/release/current/standard/papaya.css': 'dist/css/papaya.css',
      verbose: true
    }),
    buble({jsx: 'h', objectAssign: 'Object.assign'}),
    resolve({}),
    license({
      banner: 'Copyright (C) F. Hoffmann-La Roche AG, 2020.\nAuthor: stanislaw.adaszewski@roche.com\nAll Rights Reserved.',
    }),
    // uglify()
  ]
}
